/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka, Nikita Bogutzki
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include <QtWidgets>
#include "datamodel.h"
#include "treemodel.h"
#include "listmodel.h"



DataModel::DataModel( QObject *parent )
  : QAbstractItemModel( parent )
{
  QList<QVariant> rootData;
  invalidKeysList = new QList <QString>;
  rootData << "Block Structure" << "Number of Keys";
  rootItem = new TreeItem( rootData );
  invalidKeys = 0;
}

DataModel::~DataModel()
{
  delete treeModel;
  delete listModel;
  delete rootItem;
}

QVariant DataModel::data( const QModelIndex &index, int role ) const
{
  if ( !index.isValid() ) {
    return QVariant();
  }

  if ( role == Qt::ForegroundRole ) {
    if ( !isDataValid( index ) ) {
      return QVariant( QColor( Qt::red ) );
    }

    return QVariant();
  }

  if ( role != Qt::DisplayRole ) {
    return QVariant();
  }

  TreeItem *item = static_cast<TreeItem*>( index.internalPointer() );
  return item->data( index.column() );
}

bool DataModel::isDataValid( const QModelIndex &index ) const
{
  TreeItem *item = static_cast<TreeItem *>( index.internalPointer() );

  if ( index.column() <= 3 ) {
    return !item->parent()->invalidElementIndices.contains( index );
  } else {
    return !item->invalidElementIndices.contains( index );
  }
}

int DataModel::getInvalidKeys() const
{
  return invalidKeys;
}

void DataModel::setDataValidity( QModelIndex index )
{
  TreeItem *item = static_cast<TreeItem*>( index.internalPointer() );
  const QMap<QString, QVariant> &itemData = item->data( index.column() ).toMap();
  bool valid = true;

  if ( itemData["type"].toInt() == 2 ) {        // 2 corresponds to integer (for keytype "spinbox")
    valid = checkInteger( itemData );
  }
  else if ( itemData["type"].toInt() == 0 ) {   // 0 corresponds to text (for keytype "text" and "file")
    valid = checkText( itemData );
  }
  else if ( itemData["type"].toInt() == 1 ) {  // 1 corresponds to combobox
    valid = checkCombo( itemData );
  }
  else if ( itemData["type"].toInt() == 5 ) {  // 5 corresponds to decimal
    valid = checkDecimalNumber( itemData );
  }

  // counts invalid keys and saves them in a list "invalidkeylist"; necessary for enable/disable nextbutton of the keypage

  if ( valid == 1 ) {
    if ( invalidKeysList->contains( itemData["name"].toString() ) ) {
      int tempindex = invalidKeysList->indexOf( itemData["name"].toString() );
      invalidKeysList->removeAt( tempindex );
      invalidKeys--;
      emit invalidKeysChanged( this->getInvalidKeys() );
      qDebug() << "Key removed from InvalidKeyList: " << itemData["name"];
      qDebug() << "InvalidKeys from DataModel:" << this->getInvalidKeys();
    }
  }
  else {
    if ( !invalidKeysList->contains( itemData["name"].toString() ) ) {
      invalidKeysList->prepend( itemData["name"].toString() );
      invalidKeys++;
      emit invalidKeysChanged( this->getInvalidKeys() );
      qDebug() << "Key moved into InvalidKeyList: " << itemData["name"];
      qDebug() << "InvalidKeys from DataModel:" << this->getInvalidKeys();
    }
  }

  bool key_valid_up_to_now = !item->invalidElementIndices.contains( index );

  if ( ( !key_valid_up_to_now ) && valid ) {
    item->invalidElementIndices.remove( index );
    emit dataChanged( index, index );

    if ( item->invalidElementIndices.isEmpty() ) {
      index = createIndex( index.row(), 0, item );

      while ( item != rootItem ) {
        item = item->parent();
        item->invalidElementIndices.remove( index );
        emit dataChanged( index, index );

        if ( !item->invalidElementIndices.isEmpty() ) {
          break;
        } else {
          index = index.parent();
        }
      }

      emit keyInputOk( rootItem->invalidElementIndices.isEmpty() );
    }
  } else if ( key_valid_up_to_now && ( !valid ) ) {
    bool block_valid_up_to_now = item->invalidElementIndices.isEmpty();
    item->invalidElementIndices.insert( index );
    emit dataChanged( index, index );

    if ( block_valid_up_to_now ) {
      index = createIndex( index.row(), 0, item );

      while ( item != rootItem ) {
        item = item->parent();
        block_valid_up_to_now = item->invalidElementIndices.isEmpty();
        item->invalidElementIndices.insert( index );
        emit dataChanged( index, index );

        if ( !block_valid_up_to_now ) {
          break;
        } else {
          index = index.parent();
        }
      }

      emit keyInputOk( rootItem->invalidElementIndices.isEmpty() );
    }
  }
}

void DataModel::setDataValidityForAll()
{
  QModelIndex i = createIndex( 0, 0, rootItem );

  while ( i != QModelIndex() ) {
    if ( i.column() >= 3 ) {
      setDataValidity( i );
    }

    if ( i.column() < static_cast<TreeItem *>(
           i.internalPointer() )->columnCount() - 1 ) {
      i = createIndex( i.row(), i.column() + 1, i.internalPointer() );
    } else {
      i = nextIndex( i );
    }
  }
}

// function check<type>: checks if the value of the type is valid
// (in general and regarding to the Description-File, which is used)

bool DataModel::checkText( const QMap<QString, QVariant> &itemData ) const
{
  bool inputValid = true;

  if ( ( itemData.contains( "enabled" ) ) &&  ( itemData["enabled"].toString() == "true" ) ) {
    if ( !itemData.contains( "default" ) ) {
      inputValid = ( inputValid && ( itemData["value"].toString().length() > 0 ) );
    }
  }
  return inputValid;
}

bool DataModel::checkCombo( const QMap<QString, QVariant> &itemData ) const
{
  bool inputValid = true;

  if ( ( itemData.contains( "enabled" ) ) &&  ( itemData["enabled"].toString() == "true" ) ) {
    if ( !itemData.contains( "default" ) ) {
      inputValid = ( inputValid && ( itemData["value"].toString().length() > 0 ) );
    }
  }
  return inputValid;
}

bool DataModel::checkInteger( const QMap<QString, QVariant> &itemData ) const
{
  bool inputValid = true;
  int value = itemData["value"].toInt();

  if ( itemData.contains( "enabled" ) &&  ( itemData["enabled"].toString() == "true" ) ) {
    if ( !itemData.contains( "default" ) ) {
      inputValid = ( inputValid && ( itemData["value"].toString().length() > 0 ) );
    }

    if ( itemData.contains( "min" ) ) {
      int min = itemData.value( "min" ).toInt();
      inputValid = ( value >= min );
    }

    if ( itemData.contains( "max" ) ) {
      int max = itemData.value( "max" ).toInt();
      bool smaller_or_equal = ( value <= max );
      inputValid = ( inputValid && smaller_or_equal );
    }
  }
  return inputValid;
}

bool DataModel::checkDecimalNumber( const QMap<QString, QVariant> &itemData ) const
{
  bool inputValid;
  double value = itemData["value"].toDouble( &inputValid );

  if ( ( itemData.contains( "enabled" ) ) &&  ( itemData["enabled"].toString() == "true" ) ) {
    if ( !itemData.contains( "default" ) ) {
      inputValid = ( inputValid && ( itemData["value"].toString().length() > 0 ) );
    }

    if ( itemData.contains( "min" ) ) {
      inputValid = ( inputValid && ( value >= ( itemData.value( "min" ).toDouble() ) ) );
    }

    if ( itemData.contains( "max" ) ) {
      inputValid = ( inputValid && ( value <= ( itemData.value( "max" ).toDouble() ) ) );
    }

    if ( itemData.contains( "more" ) ) {
      inputValid = ( inputValid && ( value > ( itemData.value( "more" ).toDouble() ) ) );
    }

    if ( itemData.contains( "less" ) ) {
      inputValid = ( inputValid && ( value < ( itemData.value( "less" ).toDouble() ) ) );
    }
  }
  return inputValid;
}

void DataModel::countInvalidKeys( bool valid, QModelIndex index )
{
  static_cast<TreeItem *>( index.internalPointer() )
  ->data( index.column() ).toMap()["valid"] = valid;

  if ( valid ) {
    invalidKeyInputs.remove( index );
  } else {
    invalidKeyInputs.insert( index );
  }
  emit keyInputOk( invalidKeyInputs.isEmpty() );
}

QVariant DataModel::headerData( int section, Qt::Orientation orientation,
                                int role ) const
{
  if ( orientation == Qt::Horizontal && role == Qt::DisplayRole ) {
    return rootItem->data( section );
  }
  return QVariant();
}

QModelIndex DataModel::index( int row, int column, const QModelIndex &parent )
const
{
  if ( !hasIndex( row, column, parent ) ) {
    return QModelIndex();
  }

  TreeItem *parentItem;

  if ( !parent.isValid() ) {
    parentItem = rootItem;
  } else {
    parentItem = static_cast<TreeItem*>( parent.internalPointer() );
  }

  TreeItem *childItem = parentItem->child( row );

  if ( childItem ) {
    return createIndex( row, column, childItem );
  } else {
    return QModelIndex();
  }
}

QModelIndex DataModel::parent( const QModelIndex &index ) const
{
  if ( !index.isValid() ) {
    return QModelIndex();
  }

  TreeItem *childItem = static_cast<TreeItem*>( index.internalPointer() );
  TreeItem *parentItem = childItem->parent();

  if ( parentItem == rootItem ) {
    return QModelIndex();
  }
  return createIndex( parentItem->row(), 0, parentItem );
}

int DataModel::rowCount( const QModelIndex &parent ) const
{
  TreeItem *parentItem;

  if ( parent.column() > 0 ) {
    return 0;
  }

  if ( !parent.isValid() ) {
    parentItem = rootItem;
  } else {
    parentItem = static_cast<TreeItem*>( parent.internalPointer() );
  }
  return parentItem->childCount();
}

int DataModel::columnCount( const QModelIndex &parent ) const
{
  if ( parent.isValid() ) {
    return static_cast<TreeItem*>( parent.internalPointer() )->columnCount();
  } else {
    return rootItem->columnCount();
  }
}

bool DataModel::setDataNoDeps( const QModelIndex &index, const QVariant &value, int /*role*/ )
{
  if ( index.isValid() ) {
    TreeItem* item = static_cast<TreeItem*>( index.internalPointer() );
    QString block, key;
    QMap<QString, QVariant> map = item->data( index.column() ).toMap();
    block = item->data( 3 ).toString();
    key = map.value( "name" ).toString();

    if ( index.column() < 4 ) {
      item->changeData( index.column(), value );
    } else {
      if ( value.type() == QVariant::Map ) {
        item->changeData( index.column(), value );
      } else {
        map.insert( "value", value );
        item->changeData( index.column(), map );
      }
    }
    setDataValidity( index );
    emit dataChanged( index, index );
    return true;
  }

  return false;
}

bool DataModel::setData( const QModelIndex &index, const QVariant &value, int role )
{
  if ( setDataNoDeps( index, value, role ) ) {
    TreeItem* item = static_cast<TreeItem*>( index.internalPointer() );
    QString block, key;
    QMap<QString, QVariant> map = item->data( index.column() ).toMap();
    block = item->data( 3 ).toString();
    key = map.value( "name" ).toString();
    checkDep( block, key );
    return true;
  }

  return false;
}

bool DataModel::setBlockEnabled( QModelIndex i, bool b )
{
  if ( i.isValid() ) {
    TreeItem* item = static_cast<TreeItem*>( i.internalPointer() );
    item->changeData( 2, b );
    //    emit blockChanged(i);
    return true;
  }

  return false;
}

QList<QVariant> DataModel::keyDataList( QModelIndex index )
{
  QList<QVariant> list;

  if ( !index.isValid() ) {
    return list;
  }

  TreeItem *item = static_cast<TreeItem*>( index.internalPointer() );
  list.append( item->data( 0 ) ); // display name of block first

  for ( int i = 4; i < item->columnCount(); i++ ) {
    list.append( item->data( i ) );
  }

  return list;
}

QList<QModelIndex> DataModel::keyIndexList( QModelIndex index )
{
  QList<QModelIndex> list;

  if ( !index.isValid() ) {
    return list;
  }

  TreeItem *item = static_cast<TreeItem*>( index.internalPointer() );
  list.append( createIndex( index.row(), 0, item ) );

  for ( int i = 4; i <= item->columnCount(); i++ ) {
    list.append( createIndex( index.row(), i, item ) );
  }

  return list;
}

void DataModel::goIntoBlock( QDomNode read, TreeItem* parent )
{
  QDomNode m = read.firstChild();
  QDomElement elem;
  TreeItem* newItem = NULL;
  QList<QVariant> blockdata;
  blockdata << 0 << 0 << 0 << 0; //initialize array to minimum size 4

  while ( !m.isNull() ) {
    elem = m.toElement();

    if ( elem.tagName() == "name" ) {
      blockdata[3] = elem.text();

      if ( blockdata[0] == 0 ) {
        blockdata[0] = blockdata[3];
      }
    }

    if ( elem.tagName() == "displayname" ) {
      blockdata[0] = elem.text();
    }

    if ( elem.tagName() == "enabled" ) {
      if ( elem.text() == "true" ) {
        blockdata[2] = true;
      } else {
        blockdata[2] = false;
      }

      newItem = new TreeItem( blockdata, parent );
      parent->appendChild( newItem );
    }

    if ( elem.tagName() == "block" ) {
      if ( newItem == NULL ) {
        qDebug() << "TreeModel::goIntoBlock: newItem ist NULL!\n";
      }

      goIntoBlock( m, newItem );
    }

    if ( elem.tagName() == "key" ) {
      if ( newItem == NULL ) {
        qDebug() << "TreeModel::goIntoBlock: newItem ist NULL!\n";
      }

      goIntoKey( m, newItem );
    }

    m = m.nextSibling();
  }
}

void DataModel::goIntoKey( QDomNode read, TreeItem* parent )
{
  QDomNode m = read.firstChild();
  QDomElement elem;
  QMap<QString, QVariant> key;

  while ( !m.isNull() ) {
    elem = m.toElement();

    if ( elem.tagName() == "name" ) {
      key["name"] = elem.text();

      if ( !key.contains( "displayname" ) ) {
        key["displayname"] = elem.text();
      }
    }

    if ( elem.tagName() == "enabled" ) {
      key.insert( "enabled", elem.text() );
    }

    if ( elem.tagName() == "type" ) {
      // small optimization
      QString text = elem.text();

      if ( QString( "text" ) == text ) {
        key.insert( "type", 0 );
      }

      if ( QString( "combobox" ) == text ) {
        key.insert( "type", 1 );
      }

      if ( QString( "spinbox" ) == text ) {
        key.insert( "type", 2 );
      }

      if ( QString( "file" ) == text ) {
        key.insert( "type", 3 );
      }

      if ( QString( "checkbox" ) == text ) {
        key.insert( "type", 4 );
      }

      if ( QString( "decimal" ) == text ) {
        key.insert( "type", 5 );
      }
    }

    if ( elem.tagName() == "displayname" ) {
      key["displayname"] = elem.text();
    }

    if ( elem.tagName() == "defaultlist" ) {
      QDomNode n = m.firstChild();
      QDomElement f;
      QMap<QString, QVariant> currentlist;
      QString item;

      while ( !n.isNull() ) {
        f = n.toElement();
        item = f.text();
        currentlist.insert( item, QVariant() );
        n = n.nextSibling();
      }

      key.insert( "currentlist", currentlist );
    }

    if ( elem.tagName() == "min" ) { // order is critical (sequence in XSD)
      key.insert( "min", elem.text() );
    }

    if ( elem.tagName() == "max" ) {
      key.insert( "max", elem.text() );
    }

    if ( elem.tagName() == "more" ) {
      key.insert( "more", elem.text() );
    }

    if ( elem.tagName() == "less" ) {
      key.insert( "less", elem.text() );
    }

    if ( elem.tagName() == "default" ) {
      key.insert( "value", elem.text() );
    }

    if ( elem.tagName() == "info" ) {
      key.insert( "info", elem.text() );
    }

    if ( elem.tagName() == "hidedisabled" ) {
      key.insert( "hidedisabled", elem.text() );
    }

    m = m.nextSibling();
  }

  parent->appendKey( key );
}

void DataModel::intoBlock( QDomNode read )
{
  QDomNode m = read.firstChild();
  QDomElement e;
  QString myName;

  while ( !m.isNull() ) {
    e = m.toElement();

    if ( e.tagName() == "name" ) {
      myName = e.text();
    }

    if ( e.tagName() == "block" ) {
      intoBlock( m );
    }

    if ( e.tagName() == "key" ) {
      intoKey( m, myName );
    }

    m = m.nextSibling();
  }
}

void DataModel::intoKey( QDomNode read, QString blockname )
{
  QDomNode m = read.firstChild();
  QDomElement e;
  QString keyname;

  while ( !m.isNull() ) {
    e = m.toElement();

    if ( e.tagName() == "name" ) {
      keyname = e.text();
    }

    if ( e.tagName() == "dep" ) {
      intoDep( m, blockname, keyname );
    }

    m = m.nextSibling();
  }
}

void DataModel::intoDep( QDomNode read, QString blockname, QString keyname )
{
  QString trigger, target;
  trigger = blockname;
  trigger += "|";
  trigger += keyname;
  QMap< QString, QMap< QString, QMap<QString, QVariant> > > dependencies;
  QMap< QString, QMap< QString, QMap<QString, QVariant> > >::iterator dep_iter;
  QString value;
  QDomNode m, n, o;
  m = read.firstChild();
  QDomElement e, f, g, h;
  QMap< QString, QMap<QString, QVariant> > targets;
  e = m.toElement();

  if ( e.tagName() == "value" ) {
    value = e.text();
  }

  m = m.nextSibling();
  e = m.toElement();

  while ( !m.isNull() ) {
    e = m.toElement();

    if ( e.tagName() == "target" ) {
      n = m.firstChild();

      while ( !n.isNull() ) {
        f = n.toElement();

        if ( f.tagName() == "targetblock" ) {
          target = f.text();
        }

        if ( f.tagName() == "targetkey" ) {
          target += "|";
          target += f.text();
        }

        if ( f.tagName() == "action" ) {
          QMap<QString, QVariant> actions;
          o = n.firstChild();

          while ( !o.isNull() ) {
            g = o.toElement();

            if ( g.tagName() == "enableblock" ) {
              actions.insert( "enableblock", g.text() );
            }

            if ( g.tagName() == "enablekey" ) {
              actions.insert( "enablekey", g.text() );
            }

            if ( g.tagName() == "setvalue" ) {
              actions.insert( "setvalue", g.text() );
            }

            if ( g.tagName() == "setmin" ) {
              actions.insert( "setmin", g.text().toInt() );
            }

            if ( g.tagName() == "setmax" ) {
              actions.insert( "setmax", g.text().toInt() );
            }

            if ( g.tagName() == "insertitems" ) {
              QMap<QString, QVariant> itemList;
              QDomNode q = o.firstChild();
              QString item;

              while ( !q.isNull() ) {
                h = q.toElement();
                item = h.text();
                itemList.insert( item, QVariant() );
                q = q.nextSibling();
              }

              actions.insert( "insertitems", itemList );
            }

            if ( g.tagName() == "removeitems" ) {
              QMap<QString, QVariant> itemList;
              QDomNode q = o.firstChild();
              QString item;

              while ( !q.isNull() ) {
                h = q.toElement();
                item = h.text();
                itemList.insert( item, QVariant() );
                q = q.nextSibling();
              }

              actions.insert( "removeitems", itemList );
            }

            if ( g.tagName() == "checkremove" ) {
              QList<QVariant> list;
              QDomNode q = o.firstChild();

              while ( !q.isNull() ) {
                h = q.toElement();
                list.append( h.text() );
                q = q.nextSibling();
                h = q.toElement();
                list.append( h.text() );
                q = q.nextSibling();
              }

              actions.insert( "checkremove", list );
            }

            o = o.nextSibling();
          }

          targets.insert( target, actions );
        }

        n = n.nextSibling();
      }
    }

    m = m.nextSibling();
  }

  dependencies.insert( value, targets );

  if ( Dep.contains( trigger ) ) {
    QMap< QString, QMap< QString, QMap<QString, QVariant> > > deps;
    deps = Dep.value( trigger );

    for ( dep_iter = deps.begin(); dep_iter != deps.end();
          dep_iter++ ) {
      dependencies.insert( dep_iter.key(), dep_iter.value() );
    }
  }

  Dep.insert( trigger, dependencies );
}

QModelIndex DataModel::getIndexOf( QString blockandkey )
{
  QString block, key;

  if ( blockandkey.contains( "|" ) ) {
    QStringList l = blockandkey.split( "|" );
    block = l.first();
    key = l.last();
    return getIndexOf( block, key );
  } else {
    return getIndexOf( blockandkey, QString() );
  }
}

QModelIndex DataModel::getIndexOf( QString block, QString key )
{
  QModelIndex i = firstIndex();
  TreeItem* item = static_cast<TreeItem*>( i.internalPointer() );
  QString name = item->data( 3 ).toString();

  while ( ( name != block ) && ( name.remove( ' ' ) != block ) ) {
    i = nextIndex( i );

    if ( i.isValid() ) {
      item = static_cast<TreeItem*>( i.internalPointer() );
      name = item->data( 3 ).toString();
    } else {
      return QModelIndex();
    }
  }

  if ( key.isEmpty() ) {
    return createIndex( item->row(), 2, item );
  }

  int col = 4;
  QMap<QString, QVariant> map = item->data( col ).toMap();
  name = map.value( "name" ).toString();

  while ( ( name != key ) && ( name.remove( ' ' ) != key ) ) {
    col++;

    if ( col < item->columnCount() ) {
      map = item->data( col ).toMap();
      name = map.value( "name" ).toString();
    } else {
      return QModelIndex();
    }
  }

  return createIndex( item->row(), col, item );
}

void DataModel::countKeys( TreeItem* item )
{
  int i = item->columnCount() - 4;
  item->changeData( 1, i );

  for ( int j = 0; j < item->childCount(); j++ ) {
    countKeys( item->child( j ) );
  }
}

void DataModel::deleteAll()
{
  while ( rootItem->columnCount() > 2 ) {
    rootItem->removeLastColumn();
  }

  while ( !rootItem->childItems.isEmpty() ) {
    rootItem->childItems.removeLast();
  }

  Dep.clear();
}

QModelIndex DataModel::firstIndex()
{
  return createIndex( 0, 0, rootItem->child( 0 ) );
}

QModelIndex DataModel::firstEnabledIndex()
{
  QModelIndex i = createIndex( 0, 0, rootItem->child( 0 ) );

  if ( rootItem->child( 0 )->data( 2 ).toBool() ) {
    return i;
  } else {
    return nextEnabledIndex( i );
  }
}

QModelIndex DataModel::nextIndex( QModelIndex i )
{
  if ( i.isValid() ) {
    TreeItem* item = static_cast<TreeItem*>( i.internalPointer() );

    if ( item->childCount() > 0 ) {
      return createIndex( 0, 0, item->child( 0 ) );
    }

    if ( item->row() < item->rowCount() - 1 ) {
      return createIndex( item->row() + 1, 0, item->parent()->child( item->row() + 1 ) );
    }

    item = item->parent();

    while ( item != rootItem ) {
      if ( item->row() < item->rowCount() - 1 ) {
        return createIndex( item->row() + 1, 0, item->parent()->child( item->row() + 1 ) );
      }

      item = item->parent();
    }
  }

  return QModelIndex();
}

QModelIndex DataModel::nextEnabledIndex( QModelIndex i )
{
  if ( i.isValid() ) {
    TreeItem* item = static_cast<TreeItem*>( i.internalPointer() );

    // traversing children first
    for ( int j = 0; j < item->childCount(); j++ ) {
      if ( item->child( j )->data( 2 ).toBool() ) {
        return createIndex( item->child( j )->row(), 0, item->child( j ) );
      }
    }

    // then siblings
    for ( int j = item->row() + 1; j < item->rowCount(); j++ ) {
      qDebug() << "validity of sibling: " << item->parent()->child( j )->data( 2 ).toBool();

      if ( item->parent()->child( j )->data( 2 ).toBool() ) {
        return createIndex( j, 0, item->parent()->child( j ) );
      }
    }

    // then parents
    item = item->parent();

    while ( item != rootItem ) {
      for ( int j = item->row() + 1; j < item->rowCount(); j++ )
        if ( item->parent()->child( j )->data( 2 ).toBool() ) {
          return createIndex( j, 0, item->parent()->child( j ) );
        }

      item = item->parent();
    }
  }

  return QModelIndex();
}

QModelIndex DataModel::previousIndex( QModelIndex i )
{
  if ( i.isValid() ) {
    TreeItem* item = static_cast<TreeItem*>( i.internalPointer() );

    if ( item->row() > 0 ) {
      int r = item->row();
      item = item->parent()->child( r - 1 );

      while ( item->childCount() > 0 ) {
        item = item->child( item->childCount() - 1 );
      }

      return createIndex( item->row(), 0, item );
    } else {
      item = item->parent();

      if ( item != rootItem ) {
        return createIndex( item->row(), 0, item );
      }
    }
  }

  return QModelIndex();
}

QModelIndex DataModel::previousEnabledIndex( QModelIndex i )
{
  if ( i.isValid() ) {
    TreeItem* item;
    i = previousIndex( i );

    while ( i.isValid() ) {
      item = static_cast<TreeItem*>( i.internalPointer() );

      if ( item->data( 2 ).toBool() ) {
        return i;
      }

      i = previousIndex( i );
    }
  }

  return QModelIndex();
}

int DataModel::lastEnabledChild( TreeItem* item )
{
  int i = -1;

  for ( int j = 0; j < item->childCount(); j++ )
    if ( item->child( j )->data( 2 ).toBool() ) {
      i = j;
    }

  return i;
}

QTreeWidgetItem* DataModel::currentSettings()
{
  QTreeWidgetItem *root;
  root = new QTreeWidgetItem( 0 );
  root->setText( 0, "Block / Key" );
  root->setText( 1, "Value" );
  addToTree( rootItem, root );
  return root;
}

void DataModel::addToTree( TreeItem *current, QTreeWidgetItem *parent )
{
  for ( int i = 0; i < current->childCount(); i++ ) {
    if ( current->child( i )->data( 2 ).toBool() ) {
      QTreeWidgetItem *it;
      it = new QTreeWidgetItem( parent );
      it->setText( 0, current->child( i )->data( 0 ).toString() );
      addToTree( current->child( i ), it );
    }
  }

  for ( int j = 4; j < current->columnCount(); j++ ) {
    QMap<QString, QVariant> map = current->data( j ).toMap();

    if ( map.value( "enabled" ).toBool() ) {
      QTreeWidgetItem *key;
      key = new QTreeWidgetItem( parent );
      key->setText( 0, map.value( "displayname" ).toString() );

      if ( map.value( "type" ) == 1 ) {
        key->setText( 1, map.value( "currentlist" ).toMap().value( map.value( "value" ).toString() ).toString() );
      } else {
        key->setText( 1, map.value( "value" ).toString() );
      }
    }
  }
}

void DataModel::setupModelData( QDomDocument InDoc )
{
  QDomElement DocElem = InDoc.documentElement();
  QDomNode n = DocElem.firstChild();
  QDomElement e;

  while ( !n.isNull() ) {
    e = n.toElement();

    if ( e.tagName() == "block" ) {
      goIntoBlock( n, rootItem );
    }

    n = n.nextSibling();
  }

  for ( int i = 0; i < rootItem->childCount(); i++ ) {
    countKeys( rootItem->child( i ) );
  }

  // create the dependency map
  n = DocElem.firstChild();

  while ( !n.isNull() ) {
    e = n.toElement();

    if ( e.tagName() == "block" ) {
      intoBlock( n );
    }

    n = n.nextSibling();
  }

  setDataValidityForAll();
  treeModel = new TreeModel( this, this );
  listModel = new ListModel( this, firstIndex(), this );
  emit keyInputOk( rootItem->invalidElementIndices.isEmpty() );
  emit modelSetupFinished();
}

void DataModel::disableAllBlocks( TreeItem* item )
{
  QString blockname = item->data( 3 ).toString();
  setBlockEnabled( getIndexOf( blockname, QString() ), false );

  for ( int j = 0; j < item->childCount(); j++ ) {
    disableAllBlocks( item->child( j ) );
  }
}

void DataModel::setParam( QDomDocument doc )
{
  for ( int i = 0; i < rootItem->childCount(); i++ ) {
    disableAllBlocks( rootItem->child( i ) );
  }

  QDomElement docelem = doc.documentElement();
  QDomNode n = docelem.firstChild();

  while ( !n.isNull() ) {
    loadSettings( n );
    n = n.nextSibling();
  }
}

void DataModel::loadSettings( QDomNode n )
{
  QDomElement e = n.toElement();
  QModelIndex ib = getIndexOf( e.tagName(), QString() );

  if ( ib.isValid() ) {
    setBlockEnabled( ib, true );
    TreeItem* item = static_cast<TreeItem*>( ib.internalPointer() );

    for ( int i = 4; i < item->columnCount(); i++ ) {
      QMap<QString, QVariant> map = item->data( i ).toMap();
      map.insert( "enabled", false );
      QModelIndex ik = createIndex( item->row(), i, item );
      setData( ik, map );
    }

    QDomNode m = n.firstChild();

    while ( !m.isNull() ) { // check if block or key
      QDomNode o = m.firstChild();

      if ( o.nodeType() == QDomNode::ElementNode ) { // then m must be a block
        loadSettings( m );
      } else { // m is a key
        QDomElement f = m.toElement();
        QModelIndex ik = getIndexOf( e.tagName(), f.tagName() );

        if ( ik.isValid() ) {
          QMap<QString, QVariant> map = data( ik ).toMap();
          map.insert( "enabled", true );
          map.insert( "value", f.text() );
          setData( ik, map );
        } else {
          emit showInfo( "The Export File you want to load does not fit to the current Description!\nLoad the corresponding Description first!" );
        }
      }

      m = m.nextSibling();
    }
  } else {
    emit showInfo( "The Export File you want to load does not fit to the current Description!\nLoad the corresponding Description first!" );
  }
}

void DataModel::checkAllBlocks()
{
  for ( int i = 0; i < rootItem->childCount(); i++ ) {
    checkBlock( rootItem->child( i ) );
  }
}

void DataModel::checkBlock( TreeItem* item )
{
  QModelIndex ix = createIndex( item->row(), 2, item );
  //  emit blockChanged(ix);
  emit dataChanged( ix, ix );

  for ( int i = 0; i < item->childCount(); i++ ) {
    checkBlock( item->child( i ) );
  }
}

void DataModel::checkDep( QString block, QString key )
{
  QString trigger = block;
  trigger += "|";
  trigger += key;
  QMap< QString, QMap< QString, QMap< QString, QMap< QString, QVariant > > > >::iterator trigger_iter;
  QMap< QString, QMap< QString, QMap< QString, QVariant > > >::iterator value_iter;
  QMap< QString, QMap< QString, QVariant > >::iterator target_iter;
  QList< QString > targetlist;

  if ( Dep.contains( trigger ) ) { /* We have to reevaluate the deps */
    for ( trigger_iter = Dep.begin(); trigger_iter != Dep.end();
          trigger_iter++ ) {
      for ( value_iter = trigger_iter.value().begin(); value_iter !=
            trigger_iter.value().end(); value_iter++ ) {
        targetlist.append( value_iter.value().keys() );
      }
    }

    QSet< QString > tmp;
    tmp = targetlist.toSet();
    targetlist = tmp.toList();

    /*
     * For each target, iterate over the dependencies and apply the
     * relevant ones
     */
    for ( QList< QString >::const_iterator target = targetlist.constBegin();
          target != targetlist.constEnd(); target++ ) {
      applyAllDepActions( *target );
    }
  }
}

void DataModel::applyAllDepActions( QString target )
{
  QMap< QString, QMap< QString, QMap< QString, QMap< QString, QVariant > > > >::iterator triggerToValue;
  QMap< QString, QMap< QString, QVariant > >::iterator targetToAction;
  QMap<QString, QMap<QString, QMap<QString, QVariant> > > valuesToTargets;
  QMap< QString, QMap<QString, QVariant> > targetsToActions;
  QMap<QString, QVariant> actions;
  QModelIndex triggerIndex;
  TreeItem *triggerItem, *targetItem;
  QMap<QString, QVariant> triggerData, targetData;
  QString triggerValue;
  targetItem = static_cast<TreeItem*>( getIndexOf( target ).internalPointer() );
  targetData = targetItem->data( getIndexOf( target ).column() ).toMap();
  QMap<QString, QVariant> currentList = targetData.value( "currentlist" ).toMap();
  //  int min, max;
  //  min = targetData.value("defaultmin").toInt();
  //  max = targetData.value("defaultmax").toInt();
  bool target_is_block = false;
  QModelIndex targetIndex = getIndexOf( target );

  for ( triggerToValue = Dep.begin(); triggerToValue != Dep.end();
        triggerToValue++ ) {
    triggerIndex = getIndexOf( triggerToValue.key() );
    triggerItem = static_cast<TreeItem*>( triggerIndex.internalPointer() );
    triggerData = triggerItem->data( triggerIndex.column() ).toMap();
    triggerValue = triggerData.value( "value" ).toString();
    valuesToTargets = triggerToValue.value();

    if ( valuesToTargets.contains( triggerValue ) ) {
      // check if triggerValue has actions associated
      targetsToActions = valuesToTargets.value( triggerValue );

      if ( targetsToActions.contains( target ) ) {
        // execute the actions for our target
        actions = targetsToActions.value( target );

        for ( QMap<QString, QVariant>::const_iterator i =
                actions.constBegin(); i != actions.constEnd(); i++ ) {
          if ( getIndexOf( target ).column() <= 2 && i.key() ==
               "enableblock" ) { // this is a block
            target_is_block = true;

            if ( i.value().toString() == "true" ) {
              setBlockEnabled( getIndexOf( target ), true );
            } else {
              setBlockEnabled( getIndexOf( target ), false );
            }

            emit dataChanged( targetIndex, targetIndex );
          }

          if ( i.key() == "enablekey" ) {
            if ( i.value() != targetData.value( "enabled" ) ) {
              targetData.insert( "enabled", i.value() );
              emit dataChanged( targetIndex, targetIndex );
            }
          }

          if ( i.key() == "insertitems" ) {
            QMap<QString, QVariant> insertList = i.value().toMap();

            for ( QMap<QString, QVariant>::const_iterator j = insertList.constBegin();
                  j != insertList.constEnd(); ++j ) {
              currentList.insert( j.key(), j.value() );
            }
          }

          if ( i.key() == "removeitems" ) {
            QMap<QString, QVariant> removeList = i.value().toMap();

            for ( QMap<QString, QVariant>::const_iterator j = removeList.constBegin();
                  j != removeList.constEnd(); ++j ) {
              currentList.remove( j.key() );
            }
          }

          if ( i.key() == "setvalue" ) {
            if ( targetData.value( "value" ) != i.value() ) {
              targetData.insert( "value", i.value() );
            }
          }

          if ( i.key() == "setmin" ) {

            targetData.insert( "min", i.value() );
          }

          if ( i.key() == "setmax" ) {

            targetData.insert( "max", i.value() );
          }
        }
      }
    }
  }

  if ( targetData.contains( "currentlist" ) ) {
    targetData.insert( "currentlist", currentList );
  }


  if ( !target_is_block ) {
    setDataNoDeps( getIndexOf( target ), targetData, Qt::EditRole );
  }
}


void DataModel::exportBlock( QDomNode parent, TreeItem* block )
{
  if ( block->data( 2 ).toBool() ) { // falls eingeschaltet
    QDomNode name = QDomDocument().createElement( block->data( 3 ).toString().remove( ' ' ) );
    parent.appendChild( name );

    for ( int j = 3; j < block->columnCount(); j++ ) {
      QMap<QString, QVariant> map = block->data( j ).toMap();

      if ( map.value( "enabled" ).toBool() ) { // falls eingeschaltet
        QDomNode key = QDomDocument().createElement( map.value( "name" ).toString().remove( ' ' ) );
        name.appendChild( key );
        QDomText t = QDomDocument().createTextNode( map.value( "value" ).toString() );
        key.appendChild( t );
      }
    }

    for ( int i = 0; i < block->childCount(); i++ ) {
      exportBlock( name, block->child( i ) );
    }
  }
}

QDomDocument DataModel::setupOutDoc()
{
  QDomDocument OutDoc;
  QDomElement e = OutDoc.createElement( "Param" );
  OutDoc.appendChild( e );

  for ( int i = 0; i < rootItem->childCount(); i++ ) {
    exportBlock( e, rootItem->child( i ) );
  }

  return OutDoc;
}

TreeModel *DataModel::getTreeModel()
{
  return treeModel;
}

ListModel *DataModel::getListModel()
{

  return listModel;
}
