/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka, Nikita Bogutzki
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


/**
 * The TreeModel class is a model in the MVC using the QT MVC infrastructure
 * It uses the TreeItem class for storage and stores the hierachical structure
 * of the configuration data.
 *
 * The current structure is one root node of the type TreeItem. It uses a doubly
 * linked tree with one parent for each child.
 *
 * The blocks have a member itemData::QList<QVariant> with the following contents:
 * name::QString, 0::Int, enabled::bool; these are followed by one QMap per key
 * in this block.
 * That is: the structure of itemdata is:
 *  - 0 -> QString displayname, same as name if none given
 *  - 1 -> int countKeys
 *  - 2 -> bool enabled
 *  - 3 -> QString name
 *  - 4 and following -> Map for each key. These maps may have the following keys:
 *    - name
 *    - enabled
 *    - type
 *    - defaultlist
 *    - currentlist
 *    - min
 *    - max
 *    - value
 *    - info.
 *
 * Explanation is problably required for the defaultlist and currentlist;
 * defaultlist is the set of all options available.
 *
 * currentlist is the set of all options available under the current set of
 * defaults in the description file.
 *
 */

#ifndef DATAMODEL_H
#define DATAMODEL_H

#include <QtXml>
#include <QtWidgets>
#include "treeitem.h"
#include "xmlparamimport.h"

class TreeModel;
class ListModel;

/** DataModel is used for structuring the choosen XML-File with QList, QVariant, QModelIndex.
* Validations and different data types of key values are discribed here.
*/
class DataModel : public QAbstractItemModel {
private:
  Q_OBJECT

  ///counts the number of Invalidkeys; necessary for enalbe/disable next-button of the keypage
  int invalidKeys;
  QList <QString> *invalidKeysList;
  friend class TreeModel;
  friend class ListModel;
  QVariant headerData( int section, Qt::Orientation orientation,
                       int role = Qt::DisplayRole ) const;
  QModelIndex index( int row, int column, const QModelIndex &parent = QModelIndex() ) const;
  QModelIndex parent( const QModelIndex &index ) const;
  int rowCount( const QModelIndex &parent = QModelIndex() ) const;
  int columnCount( const QModelIndex &parent = QModelIndex() ) const;
  bool setBlockEnabled( QModelIndex i, bool b );
  TreeItem *rootItem;
  TreeModel *treeModel;
  ListModel *listModel;
  /**
   * Dep has the following structure:
   * Map with the keys "block|key" as the "triggers".
   *
   * Each trigger has a map associated with it in which the values for this key
   * are placed. Each of the values has a map in which
   * "targetblock|targetkey" is the key. Each of these has a map with
   * actions for this target associated with it.
   *
   * These actions might be any of: enableblock, enablekey, setvalue,
   * setmin, setmax, inseritems, removeitems and their respective values
   */
  QMap< QString, QMap< QString, QMap< QString, QMap< QString, QVariant > > > > Dep;
  void goIntoBlock( QDomNode read, TreeItem* parent ); // Tree bauen
  void goIntoKey( QDomNode read, TreeItem* parent ); // Tree bauen
  void intoBlock( QDomNode read );
  void intoKey( QDomNode read, QString blockname );
  void intoDep( QDomNode read, QString blockname, QString keyname );
  QModelIndex getIndexOf( QString blockandkey );
  QModelIndex getIndexOf( QString block, QString key );
  void applyAllDepActions( QString target );
  void exportBlock( QDomNode parent, TreeItem* block );
  void countKeys( TreeItem* item );
  void checkBlock( TreeItem* item );
  int lastEnabledChild( TreeItem* item );
  void addToTree( TreeItem *current, QTreeWidgetItem *parent );
  /**
   * Set the data at the given index for the specified role, but don't check
   * the dependencies.
   * \param index: reference to the item
   * \param value: the new value
   * \param role: the role for which to set the data (not used)
   */
  bool setDataNoDeps( const QModelIndex &index, const QVariant &value,
                      int role = Qt::EditRole );
  void loadSettings( QDomNode n );
  void disableAllBlocks( TreeItem* item );
  QSet<QModelIndex> invalidKeyInputs;
  /// function checkInteger: checks if the value of the type is valid (in general and regarding to the Description-File, which is used)
  bool checkInteger( const QMap<QString, QVariant> &itemData ) const;
  /// function checkText: checks if the value of the type is valid (in general and regarding to the Description-File, which is used)
  bool checkText( const QMap<QString, QVariant> &itemData ) const;
  /// function checkCombo: checks if the value of the type is valid (in general and regarding to the Description-File, which is used)
  bool checkCombo( const QMap<QString, QVariant> &itemData ) const;
  /// function checkDecimalNumber: checks if the value of the type is valid (in general and regarding to the Description-File, which is used)
  bool checkDecimalNumber( const QMap<QString, QVariant> &itemData ) const;
  void countInvalidKeys( bool valid, QModelIndex index );
  /// Checks, if given index is valid or invalid via bool
  void setDataValidity( QModelIndex index );
  void setDataValidityForAll();

public:
  /// Constructs the tree by initializing the root node.
  DataModel( QObject *parent );
  /// Destructor deletes the root node.
  ~DataModel();
  /**
   * Return the data associated with the index
   * \param index: reference to the item
   * \param role: the role for which data is to be returned
   */
  QVariant data( const QModelIndex &index, int role = Qt::DisplayRole ) const;
  ///return the number of InvalidKeys; necessary for enable/disable next-button of the keypage
  int getInvalidKeys()  const;
  /** Set the data at the given index for the specified role.
   * \param index: reference to the item
   * \param value: the new value
   * \param role: the role for which to set the data (not used)
   */
  bool setData( const QModelIndex &index, const QVariant &value,
                int role = Qt::EditRole );
  /**
   * check if the given QModelIndex has a value within the bounds defined by dependency actions
   * \param index: reference to the item
   */
  bool isDataValid( const QModelIndex &index ) const;
  /// Delete all the items and associated deps.
  void deleteAll();
  /**
   * Return all data associated with the indexed item
   * \param index: reference to the item
   */
  QList<QVariant> keyDataList( QModelIndex index );
  /**
   * Return all the keys associated with the indexed item
   * \param index: reference to the item
   */
  QList<QModelIndex> keyIndexList( QModelIndex index );

  QModelIndex firstIndex(); /**< Return the index to the first item */
  /**
   * Return the index following i
   * \param i: reference to the item
   */
  QModelIndex nextIndex( QModelIndex i );
  /// Return index to first enabled item.
  QModelIndex firstEnabledIndex();
  /**
   * Return the next index following i which is enabled
   * \param i: reference to the item
   */
  QModelIndex nextEnabledIndex( QModelIndex i );

  /**
   * Return the index before item i
   * \param i: reference to the item
   */
  QModelIndex previousIndex( QModelIndex i );
  /**
   * Return the next index before item i which is enabled
   * \param i: reference to the item
   */
  QModelIndex previousEnabledIndex( QModelIndex i );
  /// Create a TreeWidgetItem and return it
  QTreeWidgetItem* currentSettings();
  /// Create a DomDocument with all the data in the tree
  QDomDocument setupOutDoc();
  TreeModel *getTreeModel();
  ListModel *getListModel();
signals:
  void invalidKeysChanged( int );
  void modelSetupFinished();
  void showInfo( QString );
  void keyInputOk( bool );
public slots:
  /**
   * setup the tree according to the InDoc
   * \param InDoc: document with description
   */
  void setupModelData( QDomDocument InDoc );
  /**
   * Initialize the data with parameter set
   * \param doc: document with the data
   */
  void setParam( QDomDocument doc );
  void checkAllBlocks();
  void checkDep( QString block, QString key );
};

#endif
