/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include "keyinput.h"
#include <limits>



KeyInput::KeyInput( QWidget *parent, QVariant data, QModelIndex index )
  : QWidget( parent )
{
  myIndex = index;
  setMouseTracking( true );
  setAutoFillBackground( true );
  map = data.toMap();
  box = new QGroupBox( this );
  box->setGeometry( 10, 10, 360, 50 );
  box->setTitle( map.value( "displayname", "no name defined" ).toString() );
  QMap<QString, QVariant> currentlist;
  myInfo = map.value( "info", "No Info available for this Key." ).toString();
  t = map.value( "type" ).toInt();
  QPalette p = palette();

  switch ( t ) {
  case 0:
    lineEdit = new QLineEdit( this );
    lineEdit->setGeometry( 150, 30, 200, 20 );
    lineEdit->setText( map.value( "value" ).toString() );
    connect( lineEdit, SIGNAL( editingFinished() ), this, SIGNAL( editingFinished() ) );
    break;

  case 1:
    comboBox = new QComboBox( this );
    comboBox->setGeometry( 150, 30, 200, 20 );
    currentlist = map.value( "currentlist" ).toMap();

    if ( !currentlist.contains( map.value( "value" ).toString() ) ) {
      p.setColor( QPalette::Window, QColor( Qt::red ).lighter( 120 ) );
    }

    for ( QMap<QString, QVariant>::const_iterator i =
            currentlist.constBegin(); i != currentlist.constEnd(); i++ ) {
      comboBox->addItem( i.key(), i.value() );

      if ( map.value( "value" ) == i.key() ) {
        comboBox->setCurrentIndex( comboBox->findText( i.key() ) );
      }
    }

    connect( comboBox, SIGNAL( editingFinished() ), this, SIGNAL( editingFinished() ) );
    break;

  case 2:
    spinBox = new QSpinBox( this );
    spinBox->setGeometry( 250, 30, 100, 20 );
    spinBox->setRange( std::numeric_limits<int>::min(), std::numeric_limits<int>::max() );
    spinBox->setValue( map.value( "value" ).toInt() );
    connect( spinBox, SIGNAL( editingFinished() ), this, SIGNAL( editingFinished() ) );
    break;

  case 3:
    lineEdit = new QLineEdit( this );
    lineEdit->setGeometry( 100, 30, 200, 20 );
    browseButton = new QPushButton( "Browse...", this );
    browseButton->setGeometry( 300, 25, 80, 25 );
    lineEdit->setText( map.value( "value" ).toString() );
    connect( browseButton, SIGNAL( clicked() ), this, SLOT( browseButtonClicked() ) );
    connect( lineEdit, SIGNAL( editingFinished() ), this, SIGNAL( editingFinished() ) );
    break;

  case 4:
    checkBox = new QCheckBox( this );
    checkBox->setGeometry( 330, 10, 100, 20 );
    checkBox->setChecked( map.value( "value" ).toBool() );
    connect( checkBox, SIGNAL( editingFinished() ), this, SIGNAL( editingFinished() ) );
    break;

  default:
    break;
  }

  box->setPalette( p );
  setAccess();
}

KeyInput::KeyInput( QWidget *parent )
  : QWidget( parent )
{
  setFixedSize( 400, 70 );
  box = new QGroupBox( this );
  box->setGeometry( 10, 10, 380, 50 );
}

KeyInput::~KeyInput()
{ }

QVariant KeyInput::value()
{
  QMap<QString, QVariant> currentlist;

  switch ( t ) {
  case 0:
    return lineEdit->text();
    break;

  case 1:
    return comboBox->currentText();
    break;

  case 2:
    return spinBox->value();
    break;

  case 3:
    return lineEdit->text();
    break;

  case 4:
    return checkBox->isChecked();
    break;

  default:
    break;
  }

  return QVariant();
}

QModelIndex KeyInput::index()
{
  return myIndex;
}

void KeyInput::changeData( QVariant v )
{
  qDebug() << "changeData: " << myIndex.row() << ", " << myIndex.column() << ", "
           << myIndex.internalPointer();
  QMap<QString, QVariant> currentlist;
  QPalette p = palette();

  if ( v.type() == QVariant::Map ) {
    QMap<QString, QVariant> map = v.toMap();
    this->map = map;

    switch ( t ) {
    case 0:
      lineEdit->setText( map.value( "value" ).toString() );
      break;

    case 1:
      comboBox->clear();
      comboBox->setCurrentIndex( 0 );

      if ( !map.value( "currentlist" ).toMap().contains( map.value( "value" ).toString() ) ) {
        p.setColor( QPalette::Window, QColor( Qt::red ).lighter( 120 ) );
      }

      currentlist = map.value( "currentlist" ).toMap();

      for ( QMap<QString, QVariant>::const_iterator i =
              currentlist.constBegin(); i != currentlist.constEnd(); i++ ) {
        comboBox->addItem( i.key(), i.value() );

        if ( map.value( "value" ) == i.key() ) {
          comboBox->setCurrentIndex( comboBox->findText( i.key() ) );
        }
      }

      break;

    case 2:
      qDebug() << "changeData: min: " << map.value( "min" ).toInt() << " value: "
               << map.value( "value" ).toInt();
      spinBox->setRange( map.value( "min" ).toInt(), map.value( "max" ).toInt() );
      spinBox->setValue( map.value( "value" ).toInt() );
      break;

    case 3:
      lineEdit->setText( map.value( "value" ).toString() );
      break;

    case 4:
      checkBox->setChecked( map.value( "value" ).toBool() );
      break;

    default:
      break;
    }

    setAccess();
  } else {
    switch ( t ) {
    case 0:
      lineEdit->setText( v.toString() );
      break;

    case 1:
      currentlist = map.value( "currentlist" ).toMap();
      comboBox->setCurrentIndex( comboBox->findText( v.toString() ) );
      break;

    case 2:
      spinBox->setValue( v.toInt() );
      break;

    default:
      break;
    }
  }

  box->setPalette( p );
}

const QString KeyInput::getBoxTitle() {
  return box->title();
}

const QString KeyInput::getMyInfo() {
  return myInfo;
}

void KeyInput::browseButtonClicked()
{
  QString newFileName =
    QFileDialog::getOpenFileName( this, tr( "Directory..." ) );

  if ( !newFileName.isNull() ) {
    lineEdit->setText( newFileName );
  }
}

void KeyInput::setAccess()
{
  bool enable = map.value( "enabled", "true" ).toBool();
  qDebug() << "enabled: " << enable;
  setEnabled( enable );
  showThis = enable || ( !map.value( "hidedisabled", "false" ).toBool() );
}


void KeyInput::setWhite()
{
  qDebug() << "setWhite()\n";
  QPalette p = palette();
  p.setColor( QPalette::Window,
              QApplication::palette().color( QPalette::Window ) );
  setPalette( p );
  repaint();
}

void KeyInput::setRed()
{
  qDebug() << "setRed()\n";
  QPalette p = palette();
  p.setColor( QPalette::Window, QColor( Qt::red ).lighter( 120 ) );
  setPalette( p );
  repaint();
}


