/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka, Nikita Bogutzki
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include "keypage.h"
#include "treemodel.h"
#include "listmodel.h"
#include "keydelegate.h"
#include "keyinput.h"
#include "datamodel.h"

KeyPage::KeyPage( QWidget *parent )
  : QWizardPage( parent )
{
  treeView = new QTreeView();
  listView = new QListView( this );
  keyDelegatePage = new KeyDelegate( this );
  listView->setItemDelegate( keyDelegatePage );
  dataModel = new DataModel( this );
  // text for infoBox is set from the XML
  infoBox = new QLabel( this );
  // as text should be set variable.
  infoBox->setTextFormat( Qt::RichText );
  infoBox->setWordWrap( true );
  infoBox->setFixedHeight( 100 );
  infoBox->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
  infoBox->setAlignment( Qt::AlignTop );
  blockName = new QLabel();
  QFont blockNameFont = blockName->font();
  blockNameFont.setWeight( QFont::Bold );
  blockName->setFont( blockNameFont );
  hLayout = new QHBoxLayout();
  hLayout->addWidget( treeView );
  rLayout = new QVBoxLayout();
  rLayout->addWidget( blockName );
  rLayout->addWidget( listView );
  rLayout->addWidget( infoBox );
  hLayout->addLayout( rLayout );
  setLayout( hLayout );
  // connects with the signal of keydelegate, which includes the data shown in the infobox
  connect( listView->itemDelegate(), SIGNAL( setInfoD( QString, QString ) ), this,
           SLOT( setInfo( QString, QString ) ) );
}

KeyPage::~KeyPage() {}


void KeyPage::invalidKeysChangedBool( int invalidKeysKeyPage )
{
  if ( invalidKeysKeyPage == 0 ) {
    emit invalidKeysChangedKeyPage( true );
  }
  else {
    emit invalidKeysChangedKeyPage( false );
  }
}

void KeyPage::setInfo( QString title, QString text )
{
  QString s = "<body bgcolor='#FBB117'><h3>";
  s += title;
  s += "</h3>\n";
  s += "<h6>";
  s += text;
  s += "</h6></body>";
  infoBox->setText( s );
}

void KeyPage::setInfo( QString warning )
{
  QString s = "<h3>Warning!</h3>\n";
  s += warning;
  infoBox->setText( s );
}



void KeyPage::receiveModel( QAbstractItemModel *model )
{
  dataModel = static_cast<DataModel *>( model );
  treeModel = dataModel->getTreeModel();
  treeView->setModel( treeModel );
  treeView->setColumnWidth( 0, 280 );
  treeView->setColumnWidth( 1, 35 );
  treeView->expandAll();
  listModel = dataModel->getListModel();
  listView->setModel( listModel );
  // determines behaviour of editing listView, following status are possible:
  //QAbstractItemView::NoEditTriggers 0 No editing possible.
  //QAbstractItemView::CurrentChanged 1 Editing start whenever current item changes.
  //QAbstractItemView::DoubleClicked  2 Editing starts when an item is double clicked.
  //QAbstractItemView::SelectedClicked  4 Editing starts when clicking on an already selected item.
  //QAbstractItemView::EditKeyPressed 8 Editing starts when the platform edit key has been pressed over an item.
  //QAbstractItemView::AnyKeyPressed  16  Editing starts when any key is pressed over an item.
  //QAbstractItemView::AllEditTriggers  31  Editing starts for all above actions.
  listView->setEditTriggers( QListView::DoubleClicked );
  connect( treeView->selectionModel(),
           SIGNAL( selectionChanged( QItemSelection, QItemSelection ) ),
           treeModel, SLOT( select( QItemSelection ) ) );
  connect( treeModel, SIGNAL( selectionChanged( QItemSelection ) ),
           this, SLOT( replaceList( const QItemSelection & ) ) );
  connect( treeModel, SIGNAL( afterLastIndex() ), this,
           SIGNAL( goToNextPage() ) );
  connect( treeModel, SIGNAL( beforeFirstIndex() ), this,
           SIGNAL( goToPreviousPage() ) );
  connect( treeModel, SIGNAL( dataChanged( QModelIndex, QModelIndex ) ),
           this, SLOT( setBlockHidden( const QModelIndex & ) ) );
  treeModel->start();
  connect( dataModel, SIGNAL( invalidKeysChanged( int ) ), this, SLOT ( invalidKeysChangedBool( int ) ) );
}

void KeyPage::replaceList( const QItemSelection &selection )
{
  treeView->selectionModel()->select( selection, QItemSelectionModel::ClearAndSelect );
  QModelIndex index = selection.indexes().first();
  listModel->setParent( index );
  blockName->setText( treeModel->data( index, Qt::DisplayRole ).toString() );
  infoBox->setText( "" );   // reset infoBox text
}

void KeyPage::selectNext()
{
  treeModel->selectNext();
}

void KeyPage::selectPrevious()
{
  treeModel->selectPrevious();
}

void KeyPage::setBlockHidden( const QModelIndex &index )
{
  if ( index.column() == 2 ) {
    treeView->setRowHidden( index.row(), treeModel->parent( index ),
                            !index.data( Qt::DisplayRole ).toBool() );
  }
}
