/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka, Nikita Bogutzki
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include "opengpigui.h"
#include "xmlparamimport.h"
#include "xmlparamexport.h"

OpenGPIGUI::OpenGPIGUI( QWidget *parent, Qt::WindowFlags flags )
  : QMainWindow( parent, flags )
{
  resize( 850, 900 );
  CentralWidget = new QWidget( this );
  setCentralWidget( CentralWidget );
  createMenu();
  createWizard();
  Layout = new QGridLayout( CentralWidget );
  Layout->addWidget( Wizard, 0, 1, 1, 1 );
  CentralWidget->setLayout( Layout );

}

OpenGPIGUI::~OpenGPIGUI() {}

void OpenGPIGUI::closeEvent( QCloseEvent *close )
{
  emit this->Wizard->runpage->process->kill();
  close->accept();
}

void OpenGPIGUI::createMenu()
{
  LoadDescriptionAction = new QAction( tr( "Load Description File" ), this );
  LoadParamAction = new QAction( tr( "Load Export File" ), this );
  SaveParamAction = new QAction( tr( "Save Export File" ), this );
  QuitAction = new QAction( tr( "Quit" ), this );
  LoadDescriptionAction->setEnabled( true );
  LoadParamAction->setEnabled( false );
  SaveParamAction->setEnabled( false );
  QuitAction->setEnabled( true );
  connect( LoadDescriptionAction, SIGNAL( triggered() ), this,
           SLOT( getDescriptionFileName() ) );
  connect( LoadParamAction, SIGNAL( triggered() ), this, SLOT( getParamFileName() ) );
  connect( SaveParamAction, SIGNAL( triggered() ), this, SLOT( getOutFileName() ) );
  connect( QuitAction, SIGNAL( triggered() ), this, SLOT( close() ) );
  MenuBar = new QMenuBar( this );
  FileMenu = new QMenu( this );
  setMenuBar( MenuBar );
  FileMenu = MenuBar->addMenu( "File" );
  FileMenu->setEnabled( true );
  FileMenu->addAction( LoadDescriptionAction );
  FileMenu->addSeparator();
  FileMenu->addAction( LoadParamAction );
  FileMenu->addSeparator();
  FileMenu->addAction( SaveParamAction );
  FileMenu->addAction( QuitAction );
}

void OpenGPIGUI::createWizard()
{
  Wizard = new OpenGPIWizard( this );
  connect( Wizard, SIGNAL( descriptionFileNameChanged( QString, bool& ) ), this,
           SIGNAL( descriptionFileNameChanged( QString, bool& ) ) );
  connect( Wizard, SIGNAL( sendInFileName( QString ) ), this,
           SIGNAL( sendDescriptionFileName( QString ) ) );
  connect( Wizard, SIGNAL( paramFileNameSaveChanged( QString, bool& ) ), this,
           SIGNAL( paramFileNameSaveChanged( QString, bool& ) ) );
  connect( Wizard, SIGNAL( executableFileNameChanged( QString, bool& ) ), this,
           SIGNAL( executableFileNameChanged( QString, bool& ) ) );
  connect( Wizard, SIGNAL( sendOutFileName( QString ) ), this,
           SIGNAL( sendOutFileName( QString ) ) );
  connect( Wizard, SIGNAL( sendExecutableFileName( QString ) ), this,
           SIGNAL( sendExecutableFileName( QString ) ) );
  connect( Wizard->button( QWizard::FinishButton ), SIGNAL( clicked() ), this,
           SLOT( close() ) );
  connect( Wizard, SIGNAL( getCurrentSettings() ), this, SIGNAL( getCurrentSettings() ) );
  connect( this, SIGNAL( sendCurrentSettings( QTreeWidgetItem* ) ), Wizard,
           SLOT( showCurrentSettingsDialog( QTreeWidgetItem* ) ) );
  connect( this, SIGNAL( showInfo( QString ) ), Wizard, SLOT( showInfo( QString ) ) );
  connect( this, SIGNAL( giveDescriptionFileName( QString ) ), Wizard,
           SLOT( setInFileName( QString ) ) );

}

void OpenGPIGUI::clearGui()
{

}

void OpenGPIGUI::enableOtherActions()
{
  LoadParamAction->setEnabled( true );
}

void OpenGPIGUI::getDescriptionFileName()
{
  XMLParamImport::setCurrentDirectory();
  QString newFileName = QFileDialog::getOpenFileName( this, tr( "Load Description File" ), "./", tr( "Description Files (description_*.xml)" ) );
  bool fileNameOk;
  emit descriptionFileNameChanged( newFileName, fileNameOk );

  if ( fileNameOk ) {
    DescriptionFileName = newFileName;
    emit giveDescriptionFileName( DescriptionFileName );
  }
}

void OpenGPIGUI::getParamFileName()
{
  XMLParamImport::setCurrentDirectory();
  QString newFileName = QFileDialog::getOpenFileName( this, tr( "Load Parameter File" ), "./", tr( "XML files (*.xml)" ) );

  if ( !newFileName.isNull() ) {
    bool fileNameOk;
    emit paramFileNameLoadChanged( newFileName, fileNameOk );

    if ( fileNameOk ) {
      ParamFileName = newFileName;
      emit sendParamFileName( ParamFileName );
    } else {
      QMessageBox *messageBox = new QMessageBox( this );
      messageBox->setText( tr( "Couldn't load parameter file.\nParameter file names are of the form: \"*.xml\" without the prefix: \"description_\"." ) );
      messageBox->show();
    }
  }
}

void OpenGPIGUI::getOutFileName()
{
  XMLParamExport::setCurrentDirectory();
  QString fileName = QFileDialog::getSaveFileName( this, tr( "Save current settings" ), "./", tr( "XML files (*.xml)" ) );

  if ( !fileName.isNull() ) {
    bool fileNameOk;
    emit paramFileNameSaveChanged( fileName, fileNameOk );

    if ( fileNameOk ) {
      OutFileName = fileName;
      emit sendOutFileName( OutFileName );
    } else {
      QMessageBox *messageBox = new QMessageBox( this );
      messageBox->setText( tr( "Couldn't save parameter file.\nParameter file names are of the form: \"*.xml\" without any prefix: \"description_\"." ) );
      messageBox->show();
    }
  }
}

void OpenGPIGUI::setDescriptionFileName( QString name )
{
  emit giveDescriptionFileName( name );
}

void OpenGPIGUI::receiveModel( QAbstractItemModel* model )
{
  Wizard->receiveModel( model );
  enableOtherActions();
  emit getFirstKeys();
  Wizard->next();
}

void OpenGPIGUI::receiveInputValidity( bool valid )
{
  SaveParamAction->setEnabled( valid );
}
