/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/




#ifndef OPENGPIGUI_H
#define OPENGPIGUI_H

#include <QtWidgets>
#include <QtXml>

#include "datamodel.h"
#include "opengpitree.h"
#include "opengpiwizard.h"
/**
 * The OpenGPIGUI presents a QMainWindow with a file menu that contains entries
 * for loading and saving the parameters. It also contains a OpenGPITree, which
 * presents the block-key-structure, and a OpenGPIWizard with the input elements.
 */
class OpenGPIGUI: public QMainWindow {
  Q_OBJECT

public:

  /**
   * Constructs a OpenGPIGUI with file menu, OpenGPITree and OpenGPIWizard.
   * \param parent The parent widget.
   * \param flags Flags for specifying options of QMainWindow.
   */
  OpenGPIGUI( QWidget *parent, Qt::WindowFlags flags );

  /**
   * The destructor.
   */
  ~OpenGPIGUI();

  ///  When ever OpenGPI is being closed, closeEvent makes sure to kill the process of the RunPage.
  void closeEvent( QCloseEvent *Close );

public slots:

  void setDescriptionFileName( QString );

  /**
   * This slot passes the data model to the OpenGPITree and calls its function setProperties().
   * It makes the entries "Load Parameters" and "Save Parameters" available in the file menu.
   * It emits the signal getFirstKeys() to ask for the data of the first keys and switches the OpenGPIWizard to the KeyPage to show the
   * input elements.
   * \param model The data model for the OpenGPITree.
   */
  void receiveModel( QAbstractItemModel* model );

  //  /**
  //   * This slot calls the slot receiveNextKeys(data,index) of the OpenGPIWizard. It passes the data and indexes of the keys to be shown.
  //   * \param data List of data for the keys to be shown on the KeyPage of the OpenGPIWizard.
  //   * \param index List of the corresponding indexes of the keys.
  //   */
  //  void receiveNextKeys(QList<QVariant> data,QList<QModelIndex> index);

  //  /**
  //   * This slot calls the slot receivePreviousKeys(data,index) of the OpenGPIWizard. It passes the data and indexes of the keys to be shown.
  //   * \param data List of data for the keys to be shown on the KeyPage of the OpenGPIWizard.
  //   * \param index List of the corresponding indexes of the keys.
  //   */
  //  void receivePreviousKeys(QList<QVariant> data,QList<QModelIndex> index);

  //  /**
  //   * This slot calls the slot receiveNextKeys(data,index) of the OpenGPIWizard. It passes the data and indexes of the keys to be shown.
  //   * \param data List of data for the keys to be shown on the KeyPage of the OpenGPIWizard.
  //   * \param index List of the corresponding indexes of the keys.
  //   */
  //  void receiveKeys(QList<QVariant> data,QList<QModelIndex> index);

  void receiveInputValidity( bool valid );

signals:

  void giveDescriptionFileName( QString ); /**< This signal is emitted, if the user loads a description file via the file menu.
             It is connected to the slot setInFileName(QString) of the OpenGPIWizard.*/
  void descriptionFileNameChanged( QString, bool & );
  void sendDescriptionFileName( QString ); /**< This signal is forwarding the signal sendInFileName(QString) of the OpenGPIWizard.*/
  void paramFileNameLoadChanged( QString, bool & );
  void paramFileNameSaveChanged( QString, bool & );
  void executableFileNameChanged( QString, bool & );
  void sendParamFileName( QString ); /**< This signal is emitted, if the user loads a parameter file via the file menu.
             It passes the directory of the parameter file as a QString.*/
  void sendOutFileName( QString ); /**< This signal is emitted, if the user saves the parameters to a file via the file menu.
           It passes the directory of the file as a QString. It is also forwarding the signal
           sendOutFileName(QString) of the OpenGPIWizard.*/
  void sendExecutableFileName( QString ); /**< This signal is emitted, if the user runs an extern application via the file menu.
                   It passes the path of the file as a QString. It is also forwarding the signal
                   sendExecutableFileName(QString) of the OpenGPIWizard.*/
  void getFirstKeys(); /**< This signal is emitted, if the data model has been received via the slot
       receiveModel(QAbstractItemModel* model).*/
  void getNextKeys( QModelIndex ); /**< This signal is forwarding the signal getNextKeys(QModelIndex) of the OpenGPIWizard.*/
  void getPreviousKeys( QModelIndex ); /**< This signal is forwarding the signal getPreviousKeys(QModelIndex) of the OpenGPIWizard.*/
  void getKeys( QModelIndex ); /**< This signal is forwarding the signal getKeys(QModelIndex) of the OpenGPIWizard.*/
  void valueChanged( QVariant, QModelIndex ); /**< This signal is forwarding the signal valueChanged(QVariant,QModelIndex) of the
               OpenGPIWizard.*/
  //  void dataChanged(QModelIndex,QVariant); /**< This signal is forwarding the signal dataChanged(QModelIndex,QVariant) of the ParamData.
  //              It is connected to the slot changeData(QModelIndex,QVariant) of the OpenGPIWizard.*/
  void checkAllBlocks(); /**< This signal is forwarding the signal checkAllBlocks() of the OpenGPITree.*/
  void getCurrentSettings(); /**< This signal is forwarding the signal getCurrentSettings() of the OpenGPIWizard.*/
  void sendCurrentSettings( QTreeWidgetItem* ); /**< This signal is forwarding the signal sendCurrentSettings(QTreeWidgetItem*)
            of the ParamData. It is connected to the slot showCurrentSettingsDialog(QTreeWidgetItem*)
            of the OpenGPIWizard.*/
  void showInfo( QString ); /**< This signal is forwarding the signal showInfo(QString) of the ParamData. It is connected to the slot
          showInfo(QString) of the OpenGPIWizard.*/
  void runExecutable();

private:

  QString DescriptionFileName, ParamFileName, OutFileName;
  QAction *LoadDescriptionAction, *LoadParamAction, *SaveParamAction, *QuitAction;
  QMenuBar *MenuBar;
  QMenu *FileMenu;
  QWidget *CentralWidget;
  QGridLayout *Layout;
  OpenGPIWizard *Wizard;

  void createMenu();
  void createWizard();
  void clearGui();

private slots:

  void enableOtherActions();
  void getDescriptionFileName();
  void getParamFileName();
  void getOutFileName();
};
#endif
