/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka, Nikita Bogutzki
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include "opengpiwizard.h"
#include "datamodel.h"

OpenGPIWizard::OpenGPIWizard( QWidget *parent )
  : QWizard( parent ),
//    startpage( new StartPage( this ) ),
    loadpage( new LoadPage( this ) ),
    keypage( new KeyPage( this ) ),
    finalpage( new FinalPage( this ) ),
    runpage( new RunPage( this ) ),
    check( false ),
    lastKeyPage( false )
{
  setPage( Page_Load, loadpage );
  setPage( Page_Keys, keypage );
  setPage( Page_Final, finalpage );
  setPage( Page_Run, runpage );
  setOption( QWizard::NoCancelButton );
  setOption( QWizard::HaveCustomButton1 );
  setOption( QWizard::NoDefaultButton );
  setButtonText( QWizard::CustomButton1, "Current Settings" );
  QList<QWizard::WizardButton> list;
  list << QWizard::CustomButton1 << QWizard::Stretch << QWizard::BackButton
       << QWizard::NextButton << QWizard::FinishButton;
  setButtonLayout( list );
  button( QWizard::CustomButton1 )->setEnabled( false );
  disconnect( button( QWizard::NextButton ), SIGNAL( clicked() ), 0, 0 );
  disconnect( button( QWizard::BackButton ), SIGNAL( clicked() ), 0, 0 );
  disconnect( button( QWizard::FinishButton ), SIGNAL( clicked() ), 0, 0 );
  connect( button( QWizard::NextButton ), SIGNAL( clicked() ), this,
           SLOT( checkNext() ) );
  connect( button( QWizard::BackButton ), SIGNAL( clicked() ), this,
           SLOT( checkBack() ) );
  connect( button( QWizard::CustomButton1 ), SIGNAL( clicked() ), this,
           SIGNAL( getCurrentSettings() ) );
  connect( this, SIGNAL( finalPageNext() ), this, SLOT( getOutFileName() ) );
  connect( button( QWizard::FinishButton ), SIGNAL( clicked() ), this,
           SLOT( getExecutableFileName() ) );

  connect( loadpage, SIGNAL( descriptionFileNameChanged( QString, bool& ) ), this,
           SIGNAL( descriptionFileNameChanged( QString, bool& ) ) );
  connect( loadpage, SIGNAL( descriptionFileNameIsOK( bool ) ), this,
           SLOT( enableNextButton( bool ) ) );
  connect( finalpage, SIGNAL( paramFileNameSaveChanged( QString, bool& ) ), this,
           SIGNAL( paramFileNameSaveChanged( QString, bool& ) ) );
  connect( finalpage, SIGNAL( outFileNameIsOK( bool ) ), this,
           SLOT( enableNextButton( bool ) ) );
  // interprets the bool variable of invalidKeysChangedKeyPage( bool ) to enable/disable the next button of the keypage via eanbleNextButton( bool )
  connect( keypage, SIGNAL( invalidKeysChangedKeyPage( bool ) ), this, SLOT( enableNextButton( bool ) ) );

  QApplication::processEvents();
  emit loadpage->completeChanged();
}

OpenGPIWizard::~OpenGPIWizard() {}

void OpenGPIWizard::keyPressEvent( QKeyEvent* event )
{
  switch ( event->key() ) {
  case Qt::Key_Escape:
    event->ignore();  //in case of pressing Escape, the closeEvent will be ignored
    break;

  default:
    event->accept();
    break;
  }
}

void OpenGPIWizard::showInfo( QString s )
{
  keypage->setInfo( s );
}

void OpenGPIWizard::checkForLastKeyPage()
{
  check = true;
  emit getNextKeys( currentIndex.value( 0 ) );
}

void OpenGPIWizard::setNextButton()
{
  enableNextButton( ( !lastKeyPage ) || inputValid );
}

void OpenGPIWizard::checkNext()
{
  switch ( currentId() ) {

  case Page_Load:
    emit sendInFileName( loadpage->currentText() );
    button( QWizard::CustomButton1 )->setEnabled( true );
    break;

  case Page_Keys:
    next();
    keypage->setInfo( "", "" );
    keypage->selectNext();
    finalpage->checkOutFileName();
    break;

  case Page_Final:
    next();
    emit finalPageNext();
    break;

  default:
    break;
  }
}

void OpenGPIWizard::checkBack()
{
  switch ( currentId() ) {
  case Page_Load:
    back();
    break;

  case Page_Keys:
    back();
    keypage->setInfo( "", "" );
    keypage->selectPrevious();
    loadpage->checkDescriptionFileName();
    break;

  case Page_Final:
    back();
    emit sendKeypageDisplayed( currentIndex );
    break;

  case Page_Run:
    back();
    break;

  default:
    break;
  }
}

void OpenGPIWizard::enableNextButton( bool b )
{
  button( QWizard::NextButton )->setEnabled( b );
}

void OpenGPIWizard::enableFinishButton( bool b )
{
  button( QWizard::FinishButton )->setEnabled( b );
}


void OpenGPIWizard::setInFileName( QString s )
{
  restart();
  next();
  loadpage->setText( s );
  checkNext();
}

void OpenGPIWizard::getOutFileName()
{
  emit sendOutFileName( finalpage->getText() );
}

void OpenGPIWizard::getExecutableFileName()
{

}

void OpenGPIWizard::showCurrentSettingsDialog( QTreeWidgetItem* root )
{
  csd = new CurrentSettingsDialog( this, root );
  csd->exec();
}

void OpenGPIWizard::receiveModel( QAbstractItemModel *model )
{
  keypage->receiveModel( model );
}
