/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka, Nikita Bogutzki
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/





#ifndef OPENGPIWIZARD_H
#define OPENGPIWIZARD_H

#include <QtWidgets>
#include "datamodel.h"
#include "loadpage.h"
#include "keypage.h"
#include "finalpage.h"
#include "runpage.h"
#include "currentsettingsdialog.h"

/**
 * The OpenGPIWizard provides four wizard pages: a StartPage, a LoadPage, a KeyPage and a FinalPage.
 * It has a back button and a next button to navigate through the wizard pages.
 * The current settings button displays a user dialog with a representation of the current key values.
 */
class OpenGPIWizard : public QWizard {
  Q_OBJECT
  friend class OpenGPIGUI;

public:
  /**
   * Constructs a OpenGPIWizard with a StartPage, a LoadPage, a KeyPage and a FinalPage.
   * \param parent The parent widget.
   */
  OpenGPIWizard( QWidget *parent );

  /**
   * The destructor.
   */
  ~OpenGPIWizard();

 
  /// Locks the Escape Key so in case of pressing Escape, nothing will happen.
  void keyPressEvent( QKeyEvent* event );



public slots:

  /**
   * Puts the directory of the input file name (description or saved parameters) into the text line of the LoadPage and loads the file.
   * \param s The directory of the file as a string.
   */
  void setInFileName( QString s );

  /**
   * Displays a user dialog that presents the current parameter settings as a tree diagram of blocks and keys.
   * \param root The (invisible) root item in the tree diagram to be displayed.
   */
  void showCurrentSettingsDialog( QTreeWidgetItem* root );

  /**
   * Shows a warning message in the InfoBox of the KeyPage.
   * \param s The warning message.
   */
  void showInfo( QString s );

  void receiveModel( QAbstractItemModel * );

signals:
  void descriptionFileNameChanged( QString, bool & );
  void sendInFileName( QString ); /**< This signal is emitted, if a valid file name (description file or file with saved parameters) has been specified (via the LoadPage or the menu). It passes the directory of the file as a string.*/
  void paramFileNameSaveChanged( QString, bool & );
  void executableFileNameChanged( QString, bool & );
  void sendOutFileName( QString ); /**< This signal is emitted, if a valid file name and directory has been specified to save the current settings.*/
  void sendExecutableFileName( QString ); /**< This signal is emitted, if a valid executable file name has been specified.*/
  void getNextKeys( QModelIndex ); /**< This signal is emitted, if the user presses the next button on the KeyPage. It passes the index of the first key that is currently displayed (with the intention to get the keys of the next block).*/
  void getPreviousKeys( QModelIndex ); /**< This signal is emitted, if the user presses the next button on the KeyPage. It passes the index of the first key that is currently displayed (with the intention to get the keys of the previous block).*/
  void getCurrentSettings(); /**< This signal is emitted, if the user presses the current settings button.*/
  void valueChanged( QVariant, QModelIndex ); /**< This signal is forwarding the signal valueChanged(QVariant,QModelIndex) of the KeyPage.*/
  void sendKeypageDisplayed( QList<QModelIndex> ); /**< Send the index of the current Keypage, if there is one being displayed */
  void finalPageNext();

private:

  LoadPage *loadpage;
  KeyPage *keypage;
  FinalPage *finalpage;
  RunPage *runpage;

  enum { Page_Load, Page_Keys, Page_Final, Page_Run };

  CurrentSettingsDialog *csd;

  QList<QModelIndex> currentIndex;

  bool check;
  bool lastKeyPage;
  void checkForLastKeyPage();

  bool inputValid;

  void setNextButton();

private slots:

  void checkNext();
  void checkBack();

  void enableNextButton( bool b );
  void enableFinishButton( bool b );

  void getOutFileName();
  void getExecutableFileName();


};

#endif
