/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka, Nikita Bogutzki
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include "runpage.h"
#include "xmlparamexport.h"

RunPage::RunPage( QWidget *parent )
  : QWizardPage( parent )
{
  process = new QProcess( this );
  countExecutables = new QVariant( 0 );

  setTitle( "Execute and Simulate" );
  subtitle1 = new QLabel( "Specify the absolute directory for the Executable File." );
  subtitle2 = new QLabel( "Choose one picture for your simulation." );
  subtitle3 = new QLabel( "Click \"Start\" to run the executable/simulation.\nClick \"Stop\" to stop the executable/simulation immediately." );

  line1 = new QLineEdit();
  line1->resize( 200, 30 );
  line2 = new QLineEdit();
  line2->resize( 200, 30 );

  outputText = new QTextEdit();
  outputText->setMinimumSize(800, 200);

  button1 = new QPushButton( "Browse..." );
  button1->resize( 50, 30 );
  button2 = new QPushButton( "Execute" );
  button2->resize( 50, 30 );
  button3 = new QPushButton( "Stop" );
  button3->resize( 50, 30 );
  button2->setEnabled( true );
  button3->setEnabled( false );
  button4 = new QPushButton( "Browse..." );
  button4->resize( 50, 30 );
  button5 = new QPushButton( "Start" );
  button5->resize( 50, 30 );
  button6 = new QPushButton( "Stop" );
  button6->resize( 50, 30 );
  button5->setEnabled(false);
  button6->setEnabled(false);
  button7 = new QPushButton("Make");
  button7->resize( 50, 30 );
  button7->setEnabled(false);

  vLayout = new QVBoxLayout();
  hLayout1 = new QHBoxLayout();
  hLayout2 = new QHBoxLayout();
  hLayout3 = new QHBoxLayout();
  gridLayout1 = new QGridLayout();
  gridLayout1->addWidget( line1, 1, 0, 1, 1 );
  gridLayout1->addWidget( button1, 1, 1, 1, 1 );
  gridLayout2 = new QGridLayout();
  gridLayout2->addWidget( line2, 1, 0, 1, 1 );
  gridLayout2->addWidget( button4, 1, 1, 1, 1 );

  imageLabel = new QLabel();
  imageLabel->setMaximumSize(800, 300);
  imageLabel->setMinimumSize(800, 300);
  imageLabel->setAlignment(Qt::AlignCenter);

  groupBox = new QGroupBox(tr("Simulation Settings"));
  radio1 = new QRadioButton(tr("Selected only"));
  radio2 = new QRadioButton(tr("Last image only"));
  radio3 = new QRadioButton(tr("All"));
  radio1->setChecked(true);
  simBox = new QVBoxLayout;
  simBox->addWidget(radio1);
  simBox->addWidget(radio2);
  simBox->addWidget(radio3);
  simBox->addStretch(1);
  groupBox->setLayout(simBox);

  hLayout1->addWidget( button2 );
  hLayout1->addWidget( button7 );
  hLayout1->addWidget( button3 );
  hLayout2->addWidget( button5 );
  hLayout2->addWidget( button6 );
  hLayout3->addWidget( groupBox );
  hLayout3->addWidget( imageLabel );
  vLayout->addWidget( subtitle1 );
  vLayout->addLayout( gridLayout1 );
  vLayout->addLayout( hLayout1 );
  vLayout->addWidget( outputText );
  vLayout->addWidget( subtitle2 );
  vLayout->addLayout( gridLayout2 );
  vLayout->addLayout( hLayout2 );
  vLayout->addLayout( hLayout3 );
  vLayout->addWidget( subtitle3 );
  setLayout( vLayout );
  outputText->setReadOnly( true );

  connect( button1, SIGNAL( clicked() ), this, SLOT( getExecutableFileName() ) );
  connect( button4, SIGNAL( clicked() ), this, SLOT( getPictureFileName() ) );
  connect( button2, SIGNAL( clicked() ), this, SLOT( startExecutableFile() ) );
  connect( button7, SIGNAL( clicked() ), this, SLOT( startMakeFile() ) );
  connect( button3, SIGNAL( clicked() ), process, SLOT( kill() ) );
  connect( line2, SIGNAL( textChanged( QString ) ), this, SLOT( checkPictureFileName() ) );
  connect( line1, SIGNAL( textChanged( QString ) ), this, SLOT( checkMakeFileName() ) );
  connect( button5, SIGNAL( clicked() ), this, SLOT( startPictureFile() ) );
  connect( button6, SIGNAL( clicked() ), this, SLOT( stopPictureFile() ) );
  simulationBreak = false;
}

RunPage::~RunPage() {}

void RunPage::stopPictureFile()
{
  simulationBreak = true;
  groupBox->setEnabled(true);
  button5->setEnabled(true);
  button6->setEnabled(false);

}

void RunPage::startPictureFile()
{
  simulationBreak = false; //necessary for leaving while-loop; changed by stopPictureFile()
  groupBox->setEnabled(false);
  button5->setEnabled(false);
  button6->setEnabled(true);

  if (radio3->isChecked())
  {
    QDir dir( QFileInfo( line2->text() ).absoluteDir().absolutePath() );
    QDir::setCurrent( QFileInfo( line2->text() ).absoluteDir().absolutePath() );

    while (simulationBreak == false)
    {
      // Reading picture files in current directory and saving file names as a vector of QStrings
      QStringList files = dir.entryList(QDir::Files, QDir::Name);
      QVector<QString> filesVect = files.toVector();

      for (int i = 0; i < files.length(); i++)
      {
        // Initialize pictures into OpenGPI, scaling and saving them into imageLabel 
        QPixmap image(files[i]);
        image = image.scaled(800, 300, Qt::KeepAspectRatio);
        imageLabel->setPixmap(image);

        // necessary procedure for refreshing outputText while simulating
        if ( process->state() == 2 ) {
          QString outputString( "" );
          outputString.append( process->readAllStandardOutput() ); //reads StandardOutput as a String into outpuString
          outputString.append( process->readAllStandardError() ); //reads StandardError as a String into outpuString

          if ( outputString != "" ) {
            outputString.resize( outputString.length() - 1 );
            outputText->append( outputString );
            outputString.clear();
          }
        }
        // regulating the frame rate of simulation; ~20 fps for choosing 50 ms
        QTest::qWait(50);
        if (simulationBreak == true)
        {
          break;
        }
      }
      QApplication::processEvents();
    }
  }

  if (radio2->isChecked())
  {
    QDir dir( QFileInfo( line2->text() ).absoluteDir().absolutePath() );
    QDir::setCurrent( QFileInfo( line2->text() ).absoluteDir().absolutePath() );

    while (simulationBreak == false)
    {
      QStringList files = dir.entryList(QDir::Files, QDir::Name);
      QVector<QString> filesVect = files.toVector();
      QPixmap image(files[files.length()-1]);
      image = image.scaled(800, 300, Qt::KeepAspectRatio);
      imageLabel->setPixmap(image);
      if ( process->state() == 2 ) {
        QString outputString( "" );
        outputString.append( process->readAllStandardOutput() ); //reads StandardOutput as a String into outpuString
        outputString.append( process->readAllStandardError() ); //reads StandardError as a String into outpuString

        if ( outputString != "" ) {
          outputString.resize( outputString.length() - 1 );
          outputText->append( outputString );
          outputString.clear();
        }
      }
      QTest::qWait(50);
      if (simulationBreak == true)
      {
        break;
      }

    }
  }

  if (radio1->isChecked())
  {
    QPixmap image(line2->text());
    image = image.scaled(800, 300, Qt::KeepAspectRatio);
    QApplication::processEvents();
    imageLabel->setPixmap(image); 
  }
}


void RunPage::startExecutableFile()
{

  QDir::setCurrent( QFileInfo( line1->text() ).absoluteDir().absolutePath() );
  int count = countExecutables->toInt() + 1;
  countExecutables->setValue( count ); // counting, how many times an exectutable was started
  outputText->append( ">> [" + countExecutables->toString() + "] OpenGPI started executable: " + this->line1->text() + " <<" );
  QString outputString( "" );
  process->start( this->line1->text() );
  process->waitForStarted();

  // if process has been succesfully started, then enable/disable start/stop-buttons
  if ( process->state() == 2 ) {
    button3->setEnabled( true );
    button2->setEnabled( false );
  }
  else {
    outputText->append( "Executable is invalid!" );
  }

  //  procedure for refreshing outputText
  while ( process->state() == 2 ) {
    process->waitForFinished( 100 );
    outputString.append( process->readAllStandardOutput() ); //reads StandardOutput as a String into outputString
    outputString.append( process->readAllStandardError() ); //reads StandardError as a String into outputString

    if ( outputString != "" ) {
      outputString.resize( outputString.length() - 1 );
      outputText->append( outputString );
      outputString.clear();
    }

    QApplication::processEvents();
  }

  button3->setEnabled( false );
  button2->setEnabled( true );
  outputText->append( ">> [" + countExecutables->toString() + "] OpenGPI finished executable: " + this->line1->text() + " <<" );
  outputText->moveCursor( QTextCursor::End );
}

void RunPage::startMakeFile()
{

  QDir::setCurrent( QFileInfo( line1->text() ).absoluteDir().absolutePath() );
  int count = countExecutables->toInt() + 1;
  countExecutables->setValue( count ); // counting, how many times an exectutable was started
  outputText->append( ">> [" + countExecutables->toString() + "] OpenGPI started Makefile: " + this->line1->text() + " <<" );
  QString outputString( "" );

  process->setWorkingDirectory( QFileInfo( line1->text() ).absoluteDir().absolutePath() );
  process->start( "make" );
  process->waitForStarted();

  if ( process->state() == 2 ) {
    button3->setEnabled( true );
    button7->setEnabled( false );
  }
  else {
    outputText->append( "Makefile is invalid!" );
  }

  while ( process->state() == 2 ) {
    process->waitForFinished( 100 );
    outputString.append( process->readAllStandardOutput() ); //reads StandardOutput as a String into outpuString
    outputString.append( process->readAllStandardError() ); //reads StandardError as a String into outpuString

    if ( outputString != "" ) {
      outputString.resize( outputString.length() - 1 );
      outputText->append( outputString );
      outputString.clear();
    }

    QApplication::processEvents();
  }

  button3->setEnabled( false );
  button7->setEnabled( true );
  outputText->append( ">> [" + countExecutables->toString() + "] OpenGPI finished Makefile: " + this->line1->text() + " <<" );
  outputText->moveCursor( QTextCursor::End );
}

void RunPage::getPictureFileName()
{
  XMLParamExport::setCurrentDirectory();
  QFileDialog saveDialog( this, tr( "Picture file name" ), "./" );
  saveDialog.setFileMode( QFileDialog::AnyFile );
  saveDialog.setViewMode( QFileDialog::Detail );

  if ( saveDialog.exec() ) {
    line2->setText( saveDialog.selectedFiles().at( 0 ) );
  }
}

void RunPage::getExecutableFileName()
{
  XMLParamExport::setCurrentDirectory();
  QFileDialog saveDialog( this, tr( "Executable file name" ), "./" );
  saveDialog.setFileMode( QFileDialog::AnyFile );
  saveDialog.setViewMode( QFileDialog::Detail );

  if ( saveDialog.exec() ) {
    line1->setText( saveDialog.selectedFiles().at( 0 ) );
  }
}

void RunPage::checkMakeFileName()
{
  if ( QFileInfo( line1->text() ).fileName() == "Makefile" ) {
    button7->setEnabled( true );
    button2->setEnabled( false );
  }
  else {
    button7->setEnabled( false );
    button2->setEnabled( true );
  }
}

void RunPage::checkPictureFileName()
{
  if ( QFileInfo( line2->text() ).exists()) {
    button5->setEnabled( true );
  }
  else {
    button5->setEnabled( false );
  }
}

void RunPage::checkExecutableFileName()
{
  if ( QFileInfo( line1->text() ).exists() ) {
    button2->setEnabled( true );
  }
  else {
    button2->setEnabled( false );
  }
}
