/*  This file is part of the OpenGPI library
 *
 *  Copyright (C) 2009, 2010 Jonas Fietz, Mathias J. Krause, Martin Wlotzka
 *  E-mail contact: info@opengpi.net
 *  The most recent release of OpenGPI can be downloaded at
 *  <http://www.opengpi.org/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include "treemodel.h"

TreeModel::TreeModel( DataModel *model, QObject *parent )
  : QAbstractItemModel( parent ), dataModel( model )
{
  connect( dataModel, SIGNAL( dataChanged( QModelIndex, QModelIndex ) ),
           this, SLOT( sendDataChanged( const QModelIndex & ) ) );
}

QModelIndex TreeModel::index( int row, int column, const QModelIndex &parent ) const
{
  TreeItem *parentItem;

  if ( parent.isValid() ) {
    parentItem = static_cast<TreeItem *>( parent.internalPointer() );
  } else {
    parentItem = dataModel->rootItem;
  }

  return createIndex( row, column, parentItem->child( row ) );
}

QModelIndex TreeModel::parent( const QModelIndex &child ) const
{
  if ( child.isValid() ) {
    TreeItem *parentItem = static_cast<TreeItem *>( child.internalPointer() )->parent();

    if ( parentItem == 0 ) {
      return QModelIndex();
    } else {
      return createIndex( parentItem->row(), 0, parentItem );
    }
  } else {
    return QModelIndex();
  }
}

QVariant TreeModel::data( const QModelIndex &index, int role ) const
{
  TreeItem *item;

  if ( !index.isValid() ) {
    item = dataModel->rootItem;
  } else {
    item = static_cast<TreeItem *>( index.internalPointer() );
  }

  if ( role == Qt::DisplayRole ) {
    return item->data( index.column() );
  }

  if ( role == Qt::ForegroundRole ) {
    if ( dataModel->isDataValid( dataModel->createIndex( index.row(), index.column(), item ) ) ) {
      return QVariant();
    } else {
      return QColor( Qt::red );
    }
  }

  return QVariant();
}

int TreeModel::rowCount( const QModelIndex &parent ) const
{
  TreeItem *item;

  if ( parent.isValid() ) {
    item = static_cast<TreeItem *>( parent.internalPointer() );
  } else {
    item = dataModel->rootItem;
  }

  return item->childCount();
}

int TreeModel::columnCount( const QModelIndex &/*parent*/ ) const
{
  return 2;
}

QVariant TreeModel::headerData( int section, Qt::Orientation orientation,
                                int role ) const
{
  if ( orientation == Qt::Horizontal && role == Qt::DisplayRole ) {
    return dataModel->rootItem->data( section );
  } else {
    return QVariant();
  }
}

QModelIndex TreeModel::treeIndex( const QModelIndex &dataIndex ) const
{
  return createIndex( dataIndex.row(), dataIndex.column(),
                      dataIndex.internalPointer() );
}

QModelIndex TreeModel::firstEnabledIndex() const
{
  return treeIndex( dataModel->firstEnabledIndex() );
}

QModelIndex TreeModel::nextEnabledIndex(
  const QModelIndex &currentIndex ) const
{
  return treeIndex( dataModel->nextEnabledIndex( currentIndex ) );
}

QModelIndex TreeModel::previousEnabledIndex(
  const QModelIndex &currentIndex ) const
{
  return treeIndex( dataModel->previousEnabledIndex( currentIndex ) );
}

void TreeModel::select( const QItemSelection &selection )
{
  if ( !selection.isEmpty() ) {
    selected = selection;
    emit selectionChanged( selected );
  }
}

void TreeModel::start()
{
  selected.clear();
  QModelIndex first = firstEnabledIndex();
  selected.select( first, createIndex( first.row(), 1, first.internalPointer() ) );
  emit selectionChanged( selected );
}

void TreeModel::selectNext()
{
  QModelIndex nextIndex = nextEnabledIndex( selected.indexes().first() );

  if ( nextIndex.isValid() ) {
    selected.clear();
    selected.select( createIndex( nextIndex.row(), 0, nextIndex.internalPointer() ),
                     createIndex( nextIndex.row(), 1, nextIndex.internalPointer() ) );
    emit selectionChanged( selected );
  } else {
    emit afterLastIndex();
  }
}

void TreeModel::selectPrevious()
{
  QModelIndex previousIndex = previousEnabledIndex( selected.indexes().first() );

  if ( previousIndex.isValid() ) {
    selected.clear();
    selected.select( createIndex( previousIndex.row(), 0, previousIndex.internalPointer() ),
                     createIndex( previousIndex.row(), 1, previousIndex.internalPointer() ) );
    emit selectionChanged( selected );
  } else {
    emit beforeFirstIndex();
  }
}

void TreeModel::sendDataChanged( const QModelIndex &index )
{
  if ( index.column() <= 3 ) {
    QModelIndex topLeft = treeIndex( index );
    QModelIndex bottomRight = createIndex( topLeft.row(), topLeft.column() + 1,
                                           topLeft.internalPointer() );
    emit dataChanged( topLeft, bottomRight );
  }
}
